Package: gmgm
Type: Package
Title: Gaussian Mixture Graphical Model Learning and Inference
Version: 1.1.3
Authors@R: c(
    person("Jérémy", "Roos", email = "jeremy.roos@gmail.com",
    role = c("aut", "cre", "cph")),
    person("RATP Group", role = c("fnd", "cph")))
Description: Gaussian mixture graphical models include Bayesian networks and
    dynamic Bayesian networks (their temporal extension) whose local probability
    distributions are described by Gaussian mixture models. They are powerful
    tools for graphically and quantitatively representing nonlinear dependencies
    between continuous variables. This package provides a complete framework to
    create, manipulate, learn the structure and the parameters, and perform
    inference in these models. Most of the algorithms are described in the PhD
    thesis of Roos (2018) <https://theses.hal.science/tel-01943718>.
Depends: R (>= 3.5.0)
Imports: dplyr (>= 1.0.5), ggplot2 (>= 3.2.1), purrr (>= 0.3.3), rlang
        (>= 0.4.10), stats (>= 3.5.0), stringr (>= 1.4.0), tidyr (>=
        1.0.0), visNetwork (>= 2.0.8)
Suggests: testthat (>= 2.3.2)
License: GPL-3
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.2.1
NeedsCompilation: no
Packaged: 2026-02-25 21:22:41 UTC; jerem
Author: Jérémy Roos [aut, cre, cph],
  RATP Group [fnd, cph]
Maintainer: Jérémy Roos <jeremy.roos@gmail.com>
Repository: CRAN
Date/Publication: 2026-02-25 21:40:03 UTC
Built: R 4.6.0; ; 2026-04-26 18:46:20 UTC; unix
