# Generated by roxygen2: do not edit by hand

S3method(AIC,gmbn)
S3method(AIC,gmdbn)
S3method(AIC,gmm)
S3method(BIC,gmbn)
S3method(BIC,gmdbn)
S3method(BIC,gmm)
S3method(logLik,gmbn)
S3method(logLik,gmdbn)
S3method(logLik,gmm)
S3method(summary,gmbn)
S3method(summary,gmdbn)
S3method(summary,gmm)
export(add_arcs)
export(add_nodes)
export(add_var)
export(aggregation)
export(conditional)
export(density)
export(ellipses)
export(em)
export(expectation)
export(filtering)
export(gmbn)
export(gmdbn)
export(gmm)
export(inference)
export(merge_comp)
export(network)
export(param_em)
export(param_learn)
export(particles)
export(prediction)
export(propagation)
export(relevant)
export(remove_arcs)
export(remove_nodes)
export(remove_var)
export(rename_nodes)
export(rename_var)
export(reorder)
export(sampling)
export(smem)
export(smoothing)
export(split_comp)
export(stepwise)
export(struct_em)
export(struct_learn)
export(structure)
import(dplyr)
import(ggplot2)
import(purrr)
import(stringr)
import(tidyr)
import(visNetwork)
importFrom(rlang,":=")
importFrom(stats,AIC)
importFrom(stats,BIC)
importFrom(stats,logLik)
importFrom(stats,na.omit)
importFrom(stats,rnorm)
importFrom(stats,runif)
