# EiGLasso
A scalable optimization method for learning a Kronecker-sum inverse covariance matrix. Implementations of two different optimization methods are available:

 
> Joint optimization (See the folder [EiGLasso_JMLR](EiGLasso_JMLR/))  
> Jun Ho Yoon and Seyoung Kim. EiGLasso for Scalable Sparse Kronecker-Sum Inverse Covariance Estimation. *Journal of Machine Learning Research (JMLR)*, 2022.




> Flip-flop optimization (See the folder [EiGLasso_UAI](EiGLasso_UAI/))  
> Jun Ho Yoon and Seyoung Kim. EiGLasso: Scalable Estimation of Cartesian Product of Sparse Inverse Covariance Matrices. *Proceedings of the 36th Conference on Uncertainty in Artificial Intelligence (UAI)*, PMLR volume 124, 2020.  
> [[paper](https://www.junhoyoon.com/files/507_main_paper.pdf), [talk](https://youtu.be/rteWAfivvpw)]
