README

Main TeraLasso algorithm is teralasso.m

Three experiments are included as scripts: 

experiment1lambdaSweep.m: recreates Figure 8(b) with fewer random trials (to control runtime).

experiment2numConvergence.m: An experiment testing numerical convergence of the TeraLasso with nonconvex regularization, typical of those in Figure 15 (supplement).

windExperiment.m: Experiment recreating Figure 12. Our results are shown, the BiGLasso method is included but commented out due to its excessive runtime. 
