# Real Data
Raw real data used in the experiment section in the papers.

## 1. Mouse gene expression
Mouse gene expression data (log2-normalized read counts) is available from the [website](https://palmerlab.org/protocols-data/) of the authors of the paper ["Genome wide association analysis in a mouse advanced intercross line"](https://www.nature.com/articles/s41467-018-07642-8) under "G50-56 LGxSM AIL GWAS" for three tissue types: prefrontal cortex (PFC), striatum (STR), and hippocampus (HIP). We provide the gene names for each tissue type.
* mouse_pfc_genenames: gene names for 16,357 genes for hippocampus tissue.
* mouse_str_genenames: gene names for 16,830 genes for hippocampus tissue.
* mouse_hip_genenames: gene names for 16,661 genes for hippocampus tissue.

## 2. S&amp;P 500 stock price
S&amp;P 500 stock close price data with 306 companies and 2,517 days.
* snp500_closeprice.txt: company-by-day matrix data with stock close price.
* snp500_companies.txt: stock tickers for 306 companies in the order corresponding to the price data.
